# -*- coding: utf-8 -*-
import os
from functools import partial

from yurlungur.core.wrapper import (
    YMObject, YException, _YObject, _YNode, _YAttr
)

meta = YMObject()


class YObject(_YObject):
    """base class
    >>> obj = YObject("pCone")
    >>> obj("cone")
    """

    def __init__(self, item):
        if hasattr(meta, "objExists"):
            assert meta.objExists(item), "{} not found".format(item)
            self.item = item

        if hasattr(meta, "root"):
            assert meta.node(item), "{} not found".format(item)
            self.item = item

        if hasattr(meta, "Actor"):
            assert meta.Actor(item), "{} not found".format(item)
            self.item = item

    @property
    def name(self):
        return self.item

    def __call__(self, *args, **kwargs):
        if hasattr(meta, "rename"):
            return meta.rename(self.item, *args, **kwargs)

        if hasattr(meta, "root"):
            return meta.node(self.item).setName(*args, **kwargs)

        if hasattr(meta, "Actor"):
            return meta.Actor(self.item).rename(*args, **kwargs)

    def attr(self, val, *args, **kwargs):
        if hasattr(meta, "getAttr"):
            return YAttr(
                meta.getAttr(self.name + "." + val, *args, **kwargs), self.name, val
            )

        if hasattr(meta, "root"):
            return YAttr(
                meta.node(self.name).parm(val).eval(), self.name, val
            )

        if hasattr(meta, "Actor"):
            return YAttr
            self.get_editor_property(val)

        raise YException

    def __getattr__(self, item):
        if hasattr(meta, "getAttr"):
            return YAttr(
                meta.getAttr(self.name + "." + item), self.name, item
            )

        if hasattr(meta, "root"):
            return YAttr(
                meta.node(self.name).parm(item).eval(), self.name, item
            )

        if hasattr(meta, "Actor"):
            return YAttr

        raise YException

    @property
    def attrs(self, *args, **kwargs):
        if hasattr(meta, "listAttr"):
            return meta.listAttr(*args, **kwargs)

        if hasattr(meta, "root"):
            return meta.node(self.item).parms()

        if hasattr(meta, "Actor"):
            return meta.Actor(self.name).get_editor_property()

        raise YException

    @property
    def id(self):
        if hasattr(meta, "ls"):
            return meta.ls(self.name, uuid=1)[0] or 0

        if hasattr(meta, "root"):
            return meta.node(self.name).sessionId() or 0

        if hasattr(meta, "Actor"):
            return meta.Actor(self.name).tags

        raise YException


class YNode(YObject):
    """connect-able object"""

    def __init__(self, item):
        super(YNode, self).__init__(item)
        self.item = item

    @classmethod
    def create(cls, *args, **kwargs):
        if hasattr(meta, "createNode"):
            return cls(meta.createNode(*args, **kwargs))

        if hasattr(meta, "root"):
            return cls(meta.node(cls.item).createNode(*args, **kwargs))

        raise YException

    def delete(self, *args, **kwargs):
        if hasattr(meta, "delete"):
            meta.delete(self.item, *args, **kwargs)

        if hasattr(meta, "root"):
            meta.node(self.item).destroy()

    def connect(self, *args, **kwargss):
        if hasattr(meta, "connectAttr"):
            return meta.connectAttr(*args, **kwargss)

        if hasattr(meta, "root"):
            return

        raise YException

    def disconnect(self, *args, **kwargs):
        if hasattr(meta, "disconnectAttr"):
            return meta.disconnectAttr(*args, **kwargs)

        raise YException

    def inputs(self, *args, **kwargs):
        if hasattr(meta, "listConnections"):
            return partial(meta.listConnections, s=1)(*args, **kwargs)

        if hasattr(meta, "root"):
            return meta.node(self.name).inputs()

        raise YException

    def outputs(self):
        if hasattr(meta, "listConnections"):
            return partial(meta.listConnections, d=1)(*args, **kwargs)

        if hasattr(meta, "root"):
            return meta.node(self.name).outputs()

        raise YException


class YAttr(_YAttr):
    """parametric object"""

    def __init__(self, *args, **kwargs):
        self.vals = args

    def __getitem__(self, idx):
        return self.vals[idx]

    def set(self, *args, **kwargs):
        assert len(self.vals) > 2, "parameter is invalid."
        obj, val = self.vals[1:]

        if hasattr(meta, "setAttr"):
            return meta.setAttr(obj + "." + val, *args, **kwargs)

        if hasattr(meta, "root"):
            return meta.node(obj).parm(val).set(*args, **kwargs)

        raise YException

    def add(self):
        pass

    @property
    def value(self):
        return self.vals


class YFile(object):
    """save, load and export"""

    def __init__(self, file=""):
        self.file = file

    @classmethod
    def load(cls, *args, **kwargs):
        if hasattr(meta, "file"):
            return cls(partial(meta.file, i=1)(*args, **kwargs))

        if hasattr(meta, "hipFile"):
            return cls(meta.hipFile.load(*args, **kwargs))

        raise YException

    @classmethod
    def save(cls, *args, **kwargs):
        if hasattr(meta, "file"):
            return cls(partial(meta.file, s=1)(*args, **kwargs))

        if hasattr(meta, "hipFile"):
            return cls(meta.hipFile.save(*args, **kwargs))

        raise YException

    @property
    def name(self):
        return os.path.basename(self.file)

    @property
    def path(self):
        return os.path.dirname(self.file)

    @property
    def current(cls):
        if hasattr(meta, "file"):
            return cls(meta.file(exn=1, q=1))

        if hasattr(meta, "hipFile"):
            return cls(meta.hipFile.path())

        raise YException



# -*- coding: utf-8 -*-
from yurlungur.core.proxy import *
from yurlungur.core import enviroment as env

file = cmd = object

class Command(object):
    @staticmethod
    def register(func):
        pass

    @staticmethod
    def remove(func):
        pass

    @staticmethod
    def list(cls):
        pass


def _ls(cls):
    gen = meta.ls() if hasattr(meta, "ls") else meta.root.allItems()
    return (YNode(obj) for obj in gen)


def _rm(cls, item):
    return YNode(item).delete()


def _glob(cls):
    return


def _cd(cls):
    return


def _root(cls):
    return


def _pwd(cls):
    return


def _parent(cls, *args, **kwargs):
    return


def _children(cls, *args, **kwargs):
    return


def _select(cls, *args):
    return


def _alembicImporter(cls, *args, **kwargs):
    """
    >>> f = YFile()
    >>> YFile.new_method = new_method
    >>> print f.new_method("new")
    """
    if hasattr(meta, "AbcImport"):
        return cls(meta.AbcImport(*args, **kwargs))


def _alembicExporter(cls, *args, **kwargs):
    if hasattr(meta, "AbcExport"):
        return cls(meta.AbcExport(*args, **kwargs))


def _fbxImporter(cls, *args, **kwargs):
    if hasattr(meta, "importFBX"):
        return meta.importFBX(*args, **kwargs)
    else:
        return cls(mel.eval("FBXImport -file {0};".format(*args)))


def _fbxExporter(cls, *args, **kwargs):
    return cls(mel.eval("FBXExportInAscii -v true; FBXExport -f \"{}\" -s;".format(*args)))


# Monkey-Patch
if env.Maya():
    try:
        import maya.mel
        for plugin in ["fbxmaya.mll", "AbcImport.mll", "AbcExport.mll"]:
            meta.loadPlugin(plugin, qt=1)
    except:
        pass

    file = YFile()
    YFile.abcImporter = _alembicImporter
    YFile.abcExporter = _alembicExporter
    YFile.fbxImporter = _fbxImporter
    YFile.fbxExporter = _fbxExporter

    cmd = Command()
    Command.ls = _ls
    Command.pwd = _pwd

if env.Houdini():
    file = YFile()
    YFile.fbxImporter = _fbxImporter

    cmd = Command()
    # Command.ls = _ls

if env.Unreal():
    file = YFile()
    YFile.abcImporter = _alembicImporter
    YFile.fbxImporter = _fbxImporter
    
    cmd = Command()

__all__ = ["file", "cmd", "Command"]



# -*- coding: utf-8 -*-
import sys

application = sys.executable


def exApplication(module=""):
    application = ""

    try:
        import pysbs
        application = pysbs
    except:
        pass

    if application == "":
        from yurlungur.core import standalone
        application = standalone
    return application


if "maya" in application:
    from maya import cmds, mel

    application = cmds

elif "houdini" in application or "hindie" in application:
    import hou

    application = hou

elif "UE4Editor" in application:
    import unreal

    application = unreal

elif "max" in application:
    import pymxs

    application = pymxs

else:
    application = exApplication()

__all__ = ["application", "exApplication"]



# -*- coding: utf-8 -*-
import sys
import os
import traceback
import functools
import time
import inspect
import sqlite3


def __db_loader():
    from collections import namedtuple

    local = os.path.dirname(os.path.dirname(inspect.currentframe().f_code.co_filename))
    cache = os.path.join(local, "user", "cache.db").replace(os.sep, "/")

    conn = sqlite3.connect(cache)
    c = conn.cursor()
    for row in c.execute('SELECT * FROM stocks ORDER BY price'):
        print(row)

    return namedtuple


def __db_attr():
    local = os.path.dirname(os.path.dirname(inspect.currentframe().f_code.co_filename))
    cache = os.path.join(local, "user", "cache.db").replace(os.sep, "/")

    conn = sqlite3.connect(cache)
    c = conn.cursor()
    c.execute('''CREATE TABLE stocks
                 (date text, trans text, symbol text, qty real, price real)''')
    c.execute("INSERT INTO stocks VALUES ('2006-01-05','BUY','RHAT',100,35.14)")
    conn.commit()
    conn.close()


def __import__(name, globals=None, locals=None, fromlist=None):
    # Fast path: see if the module has already been imported.
    try:
        return sys.modules[name]
    except KeyError:
        pass

    try:
        import imp
    except:
        from importlib import import_module
        return import_module(name)

    fp, pathname, description = imp.find_module(name)
    try:
        return imp.load_module(name, fp, pathname, description)
    finally:
        if fp:
            fp.close()


def __make_completer(mod):
    local = os.path.dirname(os.path.dirname(inspect.currentframe().f_code.co_filename))
    completer = os.path.join(local, "user", "completer.pyi").replace(os.sep, "/")

    header = "\"\"\"this document generated by internal module.\"\"\"\n\n\n"
    module = __import__(mod)
    with open(completer, "w") as f:
        f.write(header)

        for fn, _ in inspect.getmembers(module):
            if fn.startswith("_"):
                continue

            f.write("def {0}(*args, **kwargs):\n".format(fn))
            f.write("   \"\"\"{0}\"\"\"\n".format(inspect.getdoc(fn)))
            f.write("   pass\n\n")

    # return inspect.getmembers(__import__(completer))


def __dark_view(view):
    local = os.path.dirname(
        os.path.dirname(inspect.currentframe().f_code.co_filename)
    )
    with open(local + "/user/dark.css") as f:
        view.setStyleSheet("".join(f.readlines()))


def analyze(func):
    @functools.wraps(func)
    def Wrapper(*args, **kw):
        try:
            ret = func(*args, **kw)
            return ret
        except:
            print(traceback.format_exc())
        return

    return Wrapper


def timer(func):
    @functools.wraps(func)
    def Wrapper(*args, **kw):
        print('{0} start'.format(func.__name__))
        start_time = time.clock()
        ret = func(*args, **kw)
        end_time = time.clock()
        print('\n{0}: {1:,f}s'.format("total: ", (end_time - start_time)))
        return ret

    return Wrapper


try:
    from yurlungur.Qt.QtCore import *
    from yurlungur.Qt.QtGui import *
    from yurlungur.Qt.QtWidgets import *
except ImportError:
    pass


class __GCProtector(object):
    widgets = []


def max_protect_window(w):
    w.setWindowFlags(Qt.WindowStaysOnTopHint)
    w.show()
    __GCProtector.widgets.append(w)


def qmain_window():
    import yurlungur.core.app
    app_name = yurlungur.core.application.__name__

    if app_name == "maya.cmds":
        from yurlungur.Qt import QtCompat
        from maya import OpenMayaUI
        ptr = long(OpenMayaUI.MQtUtil.mainWindow())
        return QtCompat.wrapInstance(ptr, QWidget)

    if app_name == "pymxs":
        import MaxPlus
        return MaxPlus.QtHelpers_GetQmaxMainWindow()
        # MaxPlus.NotifyQWidgetModalityChange(w, false)
        # MaxPlus.MakeQWidgetDockable(w, 14)

    if app_name == "hou":
        import hou
        return hou.qt.mainWindow()

    return None


def show(view):
    try:
        view.deleteLater()
    except:
        pass

    try:
        view.show()
    except:
        view.deleteLater()
        traceback.print_exc()

    if not QApplication.instance():
        app = QApplication(sys.argv)
        __dark_view(view)
        view.show()
        sys.exit(app.exec_())



from __future__ import print_function, unicode_literals, absolute_import
import sys
assert sys.version_info > (2, 7), ('yurlungur currently requires Python 2.7 later')
sys.dont_write_bytecode = True

# open yurlungur
import yurlungur
from yurlungur.core import *        # noQA
# from yurlungur.tool.math import *   # noQA
from yurlungur.tool.util import *   # noQA
from yurlungur.tool.meta import meta

__all__ = []
__version__ = "0.9.1"
name = __name__
version = __version__

# logger
from logging import basicConfig, getLogger, StreamHandler, DEBUG
basicConfig(level=DEBUG)
logger = getLogger(name)
handle = StreamHandler()
logger.addHandler(handle)



# -*- coding: utf-8 -*-
import os
import sys
import platform
import functools


def Qt(func=None):
    try:
        import yurlungur.Qt as Qt
        isQt = any([Qt])
    except ImportError:
        return False

    if func == None:
        return isQt

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if isQt:
            return func(*args, **kwargs)

    return wrapper


def Numpy(func=None):
    try:
        import numpy as nm
        return True
    except ImportError:
        return False


def Windows(func=None):
    if func == None:
        return platform.system() == "Windows"

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if platform.system() == "Windows":
            return func(*args, **kwargs)

    return wrapper


def Linux(func=None):
    if func == None:
        return platform.system() == "Linux"

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if platform.system() == "Linux":
            return func(*args, **kwargs)

    return wrapper


def MacOS(func=None):
    if func == None:
        return platform.system() == "Darwin"

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if platform.system() == "Darwin":
            return func(*args, **kwargs)

    return wrapper


def Maya(func=None):
    if func == None:
        return "maya" in sys.executable

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if "maya" in sys.executable:
            return func(*args, **kwargs)

    return wrapper


def Houdini(func=None):
    if func == None:
        return "houdini" in sys.executable or "hindie" in sys.executable

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if "houdini" in sys.executable or "hindie" in sys.executable:
            return func(*args, **kwargs)

    return wrapper


def Unreal(func=None):
    if func == None:
        return "UE4" in sys.executable

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        if "UE4" in sys.executable:
            return func(*args, **kwargs)

    return wrapper


def installed(app):
    if app == "maya":
        return os.path.getsize(_Maya())
    if app == "houdini":
        return os.path.getsize(_Houdini())
    if app == "unreal":
        return os.path.getsize(_Unreal())
    if app == "blender":
        return os.path.getsize(_Blender())
    if app == "max":
        return os.path.exists(_Max())
    return False


def _Maya():
    """find Maya app"""
    d = {
        "Linux": "/usr/autodesk/maya2017-x64",
        "Windows": "C:/Program Files/Autodesk/Maya2017",
        "Darwin": "/Applications/Autodesk/maya2017/Maya.app/Contents",
    }
    return os.environ.get("MAYA_LOCATION") or d[platform.system()]


def _Houdini():
    """find Houdini app"""
    d = {
        "Linux": "/usr/autodesk/maya2017-x64",
        "Windows": "C:/Program Files/Side Effects Software/Houdini 16.5.323/bin",
        "Darwin": "/Applications/houdini/Houdini.app/Contents",
    }
    return os.environ.get("HIP") or d[platform.system()]


def _Max():
    """find 3dsMax app"""
    return os.environ.get("ADSK_3DSMAX_X64_2018") or "C:/Program Files/Autodesk/3ds Max 2018"


def _Blender():
    """find Blender app"""
    d = {
        "Linux": "",
        "Windows": "C:/Program Files/Blender Foundation/Blender",
        "Darwin": "/Applications/Blender/blender.app/Contents/MacOS/blender"
    }
    return d[platform.system()]


def _Unreal():
    d = {
        "Linux": "",
        "Windows": "C:/Program Files/Epic Games/UE_4.19/Engine/Binaries/Win64",
        "Darwin": ""
    }
    return d[platform.system()]


MayaBin = _Maya()
HoudiniBin = _Houdini()
UnrealBin = _Unreal()

__all__ = [
    "Windows", "Linux", "MacOS",
    "Maya", "Houdini", "Unreal",
    "MayaBin", "HoudiniBin", "UnrealBin"
]



# -*- coding: utf-8 -*-
import inspect
import yurlungur
from yurlungur.core import app
from yurlungur.core import enviroment as env


class YException(NotImplementedError):
    """
    >>> raise NotImplementedError(app.application)
    """
    pass


class YMObject(object):
    """command wrapper for any application"""

    def __getattr__(self, item):
        for cmd, _ in inspect.getmembers(app.application):
            if cmd == item:
                setattr(
                    yurlungur, cmd, (lambda str: dict(inspect.getmembers(app.application))[str])(cmd)
                )
                return getattr(yurlungur, item)

        return getattr(yurlungur, "")

    @property
    def module(self):
        """current application module"""
        return app.application.__name__


class ORM(object):
    def __getattr__(self, item):
        return getattr(self, item)


class MetaObject(type):
    def __new__(cls, name, bases, attrs):
        return super(MetaObject, cls).__new__(cls, name, bases, attrs)


class MetaAttr(type):
    _node = ''
    _attr = ''

    def __new__(cls, name, bases, attrs):
        return super(MetaAttr, cls).__new__(cls, name, bases, attrs)

    # def __new__(cls, *args, **kwds):
    #     if len(args) == 2:
    #         cls._node = args[0]
    #         cls._attr = args[1]
    #     elif len(args) == 1:
    #         cls._node, cls._attr = args[0].split('.')
    #     return super(MetaAttr, cls).__new__(cls, cls._node + "." + cls._attr)


class MetaNode(type):
    def __new__(cls, name, bases, attrs):
        attrs["version"] = "0.0.1"
        return super(MetaNode, cls).__new__(cls, name, bases, attrs)

    def __getattr__(self, name):
        def _(self, name):
            return name

        return _


# dynamicClass
_YObject = MetaObject("YObject", (object,), {"__doc__": MetaObject.__doc__})
_YNode = MetaNode("YNode", (object,), {"__doc__": MetaNode.__doc__})
_YAttr = MetaAttr("YAttr", (object,), {"__doc__": MetaAttr.__doc__})

_YVector = _YMatrix = _YColor = object

if env.Maya():
    import maya.api.OpenMaya as OM

    _YVector = type('_YVector', (OM.MVector,), dict())
    _YMatrix = type('_YMatrix', (OM.MMatrix,), dict())
    _YColor = type('_YColor', (OM.MColor,), dict())

elif env.Houdini() or env.Unreal():
    meta = YMObject()

    _YVector = type('_YVector', (
        meta.Vector if hasattr(meta, "Vector") else meta.Vector3,
    ), dict())

    _YMatrix = type('_YMatrix', (
        meta.Matrix if hasattr(meta, "Matrix") else meta.Matrix4,
    ), dict())

    _YColor = type('_YColor', (meta.Color,), dict())



# -*- coding: utf-8 -*-
import sys
import inspect
from math import sqrt

from yurlungur.tool.meta import meta
from yurlungur.core.wrapper import (
    _YVector, _YMatrix, _YColor
)

try:
    import maya.api.OpenMaya as OM
except ImportError:
    pass


# @total_ordering
class YVector(_YVector):
    def __init__(self):
        super(YVector, self).__init__()

    def __eq__(self, other):
        return True

    def identify(self):
        return

    def dot_poduct(self, a, b, norm=False):
        if norm:  # 正規化オプション
            a = self.normalize(a)
            b = self.normalize(b)
        dot = (a[0] * b[0]) + (a[1] * b[1])
        return dot

    def normalize(self, a):
        length = self.length(a)
        return [a[0] / length, a[1] / length]

    def length(self):
        return sqrt(self.x ** 2 + self.y ** 2)


class YMatrix(_YMatrix):
    def __init__(self):
        super(YMatrix, self).__init__()


class YColor(_YColor):
    def __init__(self):
        super(YColor, self).__init__()

        if hasattr(meta, "Color"):
            self.color = meta.Color()
        else:
            self.color = OM.MColor()

    @property
    def r(self):
        return self.color[0]

    @property
    def g(self):
        return self.color[1]

    @property
    def b(self):
        return self.color[2]


__all__ = map(lambda x: x[0], inspect.getmembers(sys.modules[__name__], inspect.isclass))



# -*- coding: utf-8 -*-
import os
import sys
import inspect
import subprocess
import tempfile
import yurlungur as yr

try:
    from yurlungur.Qt.QtCore import *
    from yurlungur.Qt.QtGui import *
    from yurlungur.Qt.QtWidgets import *
    from yurlungur.Qt import __binding__
except ImportError:
    QDockWidget = object

local = os.path.dirname(os.path.dirname(inspect.currentframe().f_code.co_filename))


def mayapy(pystr):
    assert os.path.getsize(yr.MayaBin)
    subprocess.call(
        "{0}/bin/mayapy -c \"{1};{2};{3}\"".format(
            yr.MayaBin, "import maya.standalone;maya.standalone.initialize(name='python')",
            "import sys; sys.path.append('{0}');".format(local) + pystr, "maya.standalone.uninitialize()"
        )
    )


def hython(pystr):
    assert os.path.getsize(yr.HoudiniBin)
    subprocess.call(
        "{0}/hython -c\"import sys; sys.path.append('{1}');{2}\"".format(yr.HoudiniBin, local, pystr)
    )


def maxpy(pystr):
    assert os.path.getsize(yr.MaxBin)
    subprocess.call(
        "{0}/3dsmaxpy -c \"{1};{2}\"".format(yr.MaxBin, sys.path.append(yr), pystr)
    )


def bpython(pystr):
    assert sys.version_info > (3, 5, 3), ('blender requires Python 3.5.3') or os.path.getsize(yr.BlenderBin)
    subprocess.call(
        "{0}.blender --python-expr {1} -b".format(yr.BlenderBin, pystr)
    )


def uepython(project, pystr):
    assert os.path.getsize(yr.UnrealBin) or os.path.exists(project)

    # temp
    with tempfile.NamedTemporaryFile(delete=False) as tf:
        tf.write(pystr)
    subprocess.call(
        "{0}/UE4Editor-Cmd {1} ExecutePythonScript = {2}".format(yr.UnrealBin, project, pyfile)
    )


class YurPrompt(QDockWidget):
    def __init__(self, parent=None):
        super(YurPrompt, self).__init__(parent)

        self.setWindowTitle("{0} v{2} {1}".format(yr.name, yr.application.__name__, yr.version))
        self.setWindowFlags(Qt.Window)
        self.setAttribute(Qt.WA_DeleteOnClose)
        # print os.path.join(os.path.dirname(sys.executable),
        #                    'Lib',
        #                    'site-packages',
        #                    __binding__,
        #                    'plugins',
        #                    'imageformats')
        self.init_widget()

    def init_widget(self):
        self.box_application = QComboBox()
        self.box_application.currentIndexChanged[str].connect(self.refresh_item)

        self.box_versions = QComboBox()
        self.btn_python = QPushButton("...")
        hLayout = QHBoxLayout()
        for w in [self.box_application, self.box_versions, self.btn_python]:
            hLayout.addWidget(w)

        self.text_edit = QTextEdit()
        vLayout = QVBoxLayout()
        for layout in [self.text_edit]:
            vLayout.addWidget(layout)

        self.status_bar = QStatusBar()
        sLayout = QVBoxLayout()
        self.status_bar.showMessage("Ready...")
        for layout in [self.status_bar]:
            sLayout.addWidget(self.status_bar)

        widget = QWidget()
        Alayout = QVBoxLayout()
        for layout in [hLayout, vLayout, sLayout]:
            Alayout.addLayout(layout)
        widget.setLayout(Alayout)
        self.setWidget(widget)

        self.init_attrs()
        # yr.__dark_view(self)

    def init_attrs(self):
        tmp = []
        for app in list(set(tmp)):
            self.box_application.addItem(app)

        self.box_versions.addItem("aaa")
        # sys.stdout.write("\r%d" % 1)
        sys.stdout.flush()

    def refresh_item(self):
        pass

    def reflesh_msg(self):
        self.status_bar.showMessage("")


def _cli(args):
    """
    command line parser
    """
    try:
        import argparse
    except ImportError:
        yr.logger.warn("argparse is not found.")
        sys.exit(1)

    parser = argparse.ArgumentParser(
        prog='yr.cli',
        usage='Demonstration of argparser',  # プログラムの利用方法
        description='description',  # 引数のヘルプの前に表示
        epilog="{0} v.{1} {2}".format(yr.name, yr.version, sys.executable),  # 引数のヘルプの後で表示
        add_help=True,
    )
    parser.add_argument("--dialog", "-d",
                        help="Launch yr.widget if Qt is installed.",
                        action="store_true")
    parser.add_argument("--mayapy", "-ma",
                        help="Run Python from mayapy.",
                        nargs=1)
    parser.add_argument("--hython", "-hou",
                        help="Run Python from hython.",
                        nargs=1)
    parser.add_argument("--unrealpy", "-ue",
                        help="Run Python from unreal editor cmd.",
                        nargs=2)
    parser.add_argument("--tests",
                        help="Read from stdin instead of file",
                        action="store_true")

    args = parser.parse_args(args)
    if args.dialog:
        main()

    if args.mayapy:
        mayapy(args.mayapy[0])

    if args.hython:
        hython(args.hython[0])

    if args.unrealpy:
        project, expr = args.unrealpy
        uepython(project, expr)

    if args.tests:
        pass


def main(args=[]):
    if yr.Qt():
        app = QApplication(args)
        widget = YurPrompt()
        widget.show()
        sys.exit(app.exec_())
    else:
        yr.logger.warn("Qt isn't available")


if __name__ == '__main__':
    main(sys.argv[1:])

__all__ = map(lambda x: x[0], inspect.getmembers(sys.modules[__name__], inspect.isclass))



from yurlungur.core.app import application
from yurlungur.core.builder import Builder
from yurlungur.core.command import file, cmd
from yurlungur.core.proxy import YObject, YFile, YNode
from yurlungur.core.enviroment import (
    installed, Maya, Houdini, Unreal, Qt,
    MayaBin, HoudiniBin, UnrealBin,
)